local LAM = LibAddonMenu2

DungeonTrialReset = {
	name = "DungeonTrialReset",
	version = 1,

	-- Time allowance for what is considered a "recent" disband or rejoin,
	-- mostly to accomodate for server response times
	allowance = 5000, -- 5 seconds

	-- Time delay between invites, to avoid dropped invites
	inviteDelays = {
		{ 200, 150 }, -- 150ms for high ping (200+)
		{ 100, 125 }, -- 125ms for medium ping (100-200)
		{   0, 100 }, -- 100ms for low ping
	},

	selfName = GetDisplayName(),
	members = { },
	disbandLeader = "",
	disbandTime = 0,
	rejoinTime = 0,
	requestTime = 0,
	requestCount = 0,

	-- Default settings
	defaults = {
		confirmationCount = 3,
		confirmationSensitivity = 1500, -- 1.5 seconds
		exitOption = 2,
	},
}
local DTR = DungeonTrialReset

local function OnAddOnLoaded( eventCode, addonName )
	if (addonName ~= DTR.name) then return end

	EVENT_MANAGER:UnregisterForEvent(DTR.name, EVENT_ADD_ON_LOADED)

	DTR.vars = ZO_SavedVars:NewAccountWide("DungeonTrialResetSavedVariables", 1, nil, DTR.defaults, nil, "$InstallationWide")

	SLASH_COMMANDS["/reformgroup"] = DTR.ReformGroupSlash
	DTR.RegisterEventHandlers()
	DTR.RegisterSettingsPanel()
end


--------------------------------------------------------------------------------
-- Entry points
--------------------------------------------------------------------------------

function DTR.ReformGroupKeybind( )
	if (IsUnitGroupLeader("player")) then
		local currentTime = GetGameTimeMilliseconds()
		if (currentTime - DTR.requestTime < DTR.vars.confirmationSensitivity) then
			DTR.requestCount = DTR.requestCount + 1
		else
			DTR.requestCount = 1
		end
		DTR.requestTime = currentTime

		if (DTR.requestCount < DTR.vars.confirmationCount) then
			CHAT_ROUTER:AddSystemMessage(string.format(GetString(SI_DTR_KEYBIND_CONFIRM), DTR.vars.confirmationCount - DTR.requestCount, DTR.vars.confirmationSensitivity / 1000))
		else
			DTR.requestCount = 0
			DTR.ReformGroup()
		end
	else
		CHAT_ROUTER:AddSystemMessage(GetString(SI_DTR_NOLEAD))
	end
end

function DTR.ReformGroupSlash( )
	if (IsUnitGroupLeader("player")) then
		if (LAM) then
			LAM.util.ShowConfirmationDialog(GetString(SI_DTR_SLASH_CONFIRM_TITLE), GetString(SI_DTR_SLASH_CONFIRM_TEXT), DTR.ReformGroup)
		else
			DTR.ReformGroup()
		end
	else
		CHAT_ROUTER:AddSystemMessage(GetString(SI_DTR_NOLEAD))
	end
end


--------------------------------------------------------------------------------
-- Core functions
--------------------------------------------------------------------------------

function DTR.ReformGroup( )
	DTR.members = { }
	for i = 1, GetGroupSize() do
		local member = GetUnitDisplayName(GetGroupUnitTagByIndex(i))
		if (member and member ~= DTR.selfName) then
			table.insert(DTR.members, member)
		end
	end

	DTR.disbandTime = GetGameTimeMilliseconds()
	GroupDisband()
end

function DTR.InviteMember( index, delay )
	if (index > 0) then
		GroupInviteByName(DTR.members[index])
	end
	if (index < #DTR.members) then
		zo_callLater(function() DTR.InviteMember(index + 1, delay) end, delay)
	end
end

function DTR.GetInviteDelay( index, delay )
	local ping = GetLatency()
	if (ping < 1) then ping = 1 end

	for _, entry in ipairs(DTR.inviteDelays) do
		if (ping >= entry[1]) then
			return entry[2]
		end
	end
end

function DTR.ShouldLeaveInstance( )
	if (DTR.vars.exitOption == 1) then
		return true
	elseif (DTR.vars.exitOption == 2 and IsRaidInProgress() and IsUnitInCombat("player")) then
		return true
	else
		return false
	end
end

function DTR.RegisterEventHandlers( )
	EVENT_MANAGER:RegisterForEvent(DTR.name, EVENT_GROUP_MEMBER_LEFT, function( eventCode, memberCharacterName, reason, isLocalPlayer, isLeader, memberDisplayName, actionRequiredVote )
		if (isLeader and reason == GROUP_LEAVE_REASON_DISBAND) then
			local currentTime = GetGameTimeMilliseconds()
			if (not isLocalPlayer) then
				-- Disband not issued by you: prep for receiving invite
				DTR.disbandLeader = memberDisplayName
				DTR.disbandTime = currentTime
			elseif (currentTime - DTR.disbandTime < DTR.allowance) then
				-- Disband issued by player: reinvite the group
				DTR.InviteMember(0, DTR.GetInviteDelay())
			end
		end
	end)

	EVENT_MANAGER:RegisterForEvent(DTR.name, EVENT_GROUP_INVITE_RECEIVED, function( eventCode, inviterCharacterName, inviterDisplayName )
		if (inviterDisplayName == DTR.disbandLeader and GetGameTimeMilliseconds() - DTR.disbandTime < DTR.allowance) then
			DTR.rejoinTime = GetGameTimeMilliseconds()
			AcceptGroupInvite()
		end
	end)

	EVENT_MANAGER:RegisterForEvent(DTR.name, EVENT_GROUP_MEMBER_JOINED, function( eventCode, memberCharacterName, memberDisplayName, isLocalPlayer )
		if (isLocalPlayer and GetGameTimeMilliseconds() - DTR.rejoinTime < DTR.allowance and DTR.ShouldLeaveInstance()) then
			zo_callLater(ExitInstanceImmediately, 500)
		end
	end)
end


--------------------------------------------------------------------------------
-- Settings panel
--------------------------------------------------------------------------------

function DTR.RegisterSettingsPanel( )
	if (not LAM) then return end

	local panelId = "DTR_Settings"

	local settingsPanel = LAM:RegisterAddonPanel(panelId, {
		type = "panel",
		name = GetString(SI_DTR_TITLE),
		author = "@code65536",
	})

	local exitOptions = { }
	local exitOptionLabels = { }
	for i = 0, 2 do
		table.insert(exitOptions, i)
		table.insert(exitOptionLabels, GetString("SI_DTR_EXIT", i))
	end

	LAM:RegisterOptionControls(panelId, {
		--------------------------------------------------------------------
		{
			type = "dropdown",
			name = SI_DTR_EXIT,
			choices = exitOptionLabels,
			choicesValues = exitOptions,
			getFunc = function() return DTR.vars.exitOption end,
			setFunc = function(option) DTR.vars.exitOption = option end,
		},
	})
end

EVENT_MANAGER:RegisterForEvent(DTR.name, EVENT_ADD_ON_LOADED, OnAddOnLoaded)
